%Lagrange interpolation test for block processing.
%
%In block processing a constant block size is desired.  However, the block sample rate
%conversion produces all the outputs from the provided inputs as it can and you can't
%necessarily get exactly the number you want.  To address this, any extra samples are
%truncated from the block and saved for the next block and the number of needed inputs
%is correspondingly compensated.

% K - lagrange filter order (K+1 tap filter)
K = 5;

% dt = output time step (relative to the input time step which is always assumed to be 1)
dt = .987;

% Ny = block size for processing
Ny = 123;

% initial conditions
x0 = zeros(1,K);
t0 = 0;
tx = [0];
py = [];
pt = [];

% for some number of blocks...
for i=1:25,
  % need to compute at least Ny-length(py) new outputs
  % number of input x samples needed
  Nx = ceil((Ny-length(py))*dt+t0);

  % get the input samples
  tx = tx(length(tx))+[1:Nx];
  x = sin(tx/211*7*2*pi);

  % rate convert the inputs
  [y t N t0 x0] = asrc_lagrange(x,dt,t0,x0,K);

  % prepend the extras from the last block
  y = [py y];
  t = [pt t];
  % truncate the block, save the truncated part for next time. and offset the saved times
  if N>Ny,
    py = y(Ny+1:N);
    pt = t(Ny+1:N)-t(N)+t0-dt-(K-1)/2;
    y = y(1:Ny);
    t = t(1:Ny);
  else,
    py = [];
    pt = [];
  end;
  % fyi print the number of extra samples that had to be saved for next time
  length(py)

  % plot input and output data
  plot([1:Nx],x);
  hold on;
  plot(t,y,'.r');
  hold off;
  pause;
end;