%Lagrange interpolation impulse response.
%
%Conceptually the interploation can be viewed as reconstruction of the input sequence
%to continuous time using the interploation filter for the reconstruction then
%resampling the continuous signal at the output rate.  The passband response of the
%interpolation filter will color the signal so ideally should have constant amplitude
%and delay in the passband.  The stopband response of the interpolation filter should
%suppress the images of the input signal so as not to alias in the subsequent
%resampling.  In other words, the interploation should approximate the ideal
%reconstruction lowpass as expected.
%
%Here, the impulse response of the filter is calculated for some arbitrarily high
%multiple N of the input sample rate as an approximation to real time.

% upsampling factor
N = 64;
dt = 1/N;

% number of frequency domain sample points within nyquist of input
% (controls plotting resolution)
np = 100;

% K - lagrange filter order (K+1 tap filter)
% loop through a bunch
for K=1:2:25,
  K

  % initial conditions
  x0 = zeros(1,K);
  t0 = 0;

  x = zeros(1,K+1);
  x(1) = 1;

  % rate convert the inputs
  [h t N t0 x0] = asrc_lagrange(x,dt,t0,x0,K);

  subplot(3,1,1);
  plot([0:length(h)-1]/N/(K+1),h);

  [H,f] = freqz(h*dt,1,N*np);

  % plot the magnitude response
  subplot(3,1,2);
  % first drop a marker at the original nyquist
  plot([pi/N pi/N],[-60 0],'g',[pi pi],[-60 0],'g');
  hold on;
  % plot dB magnitude with a -60dB floor
  plot(f,max(20*log10(abs(H)),-60));
  % zoom in on the passband with a 6dB floor
  plot(f(1:np)*N,max(20*log10(abs(H(1:np))),-60/10)*10,'r');
  hold off;

  % plot the phase response
  subplot(3,1,3);
  plot(f,angle(H));
  hold on;
  % zoom in on the passband
  plot(f(1:np)*N,angle(H(1:np)),'r');
  hold off;

  pause;
end;